function [u,alpha,delta_est] = l1tv_balancing(f,K,vnorm,b_opts,s_opts)
% [u,alpha,delta_est] = L1TV_BALANCING(f,mu,K,alpha,opts)
% balancing principle for computing a regularization parameter for l1TV 
% image restoration
% Input:  f       -- blurred and noisy data
%         K       -- blurring operator as (anonymous) function
%         vnorm   -- vector norm for gradient
%         b_opts  -- options for parameter choice; structure containing (default):
%          alpha0 -- starting value for fixed point iteration (1)
%          sigma  -- balancing parameter (1.01)
%          maxits -- maximum # of iterations in fixed point iteration (10)
%          tol    -- termination criterion: relative change in alpha (1e-2)
%         s_opts  -- options for solution algorithm
% Output: u       -- solution of primal problem (l1tv reconstruction)
%         alpha   -- chosen regularization parameter
%       delta_est -- estimated noise level
% Christian Clason (christian.clason@uni-graz.at)
% Bangti Jin       (btjin@informatik.uni-bremen.de)
% August 14, 2009

%% set parameters
[n,m] = size(f);

% Default parameters
if nargin <= 3
    alpha  = 1;             % starting value for fixed point iteration
    sigma  = 1.01;          % balancing parameter
    maxits = 10;            % maximum number of iterations
    tol    = 1e-2;          % terminate if relative change in alpha smaller
else
    alpha  = b_opts.alpha0;
    sigma  = b_opts.sigma;
    maxits = b_opts.maxits;
    tol    = b_opts.tol;
end
if nargin <= 4
    s_opts = [];
end

%% fixed point iteration for balancing principle
tic;
for iter = 1:maxits
    alpha1 = alpha;
    
    % compute u_\alpha
    u = l1tv_solve(f,K,vnorm,alpha,s_opts);
    
    % compute |\nabla u_\alpha|, Ku_\alpha 
    du = vnorm([diff(u,1,2), u(:,1)-u(:,m)],...
               [diff(u,1,1); u(1,:)-u(n,:)]);
    ku = K(u);
    
    % update alpha
    alpha = (sigma-1)*norm(ku(:)-f(:),1)/(norm(du(:),1));
    
    % terminate if relative change in alpha too small
    if (abs(alpha-alpha1)/alpha<tol)
        break;
    end
    
end
t_b = toc;

% estimate noise level
delta_est = norm(ku(:)-f(:),1)/(n*m);

%% print statistics
display(sprintf('Parameter choice: alpha = %g, delta_est = %g, iter = %g, time = %g',...
    alpha, delta_est, iter, t_b));
