function [u,v] = l1tv_solve(f,K,vnorm,alpha,opts)
% [u,v] = L1TV_SOLVE(f,mu,K,alpha,opts)
% dual ascent/descent method and continuation method for the solution of
% l1TV image restoration
% Input:  f      -- blurred and noisy data
%         K      -- blurring operator as (anonymous) function
%         vnorm  -- vector norm for dual variable
%         alpha  -- regularization parameter
%         opts   -- options; structure containing (default):
%          mu    -- starting value for continuation scheme (1)
%          q     -- reduction of mu in continuation scheme (1/10)
%          outits-- maximum # of iterations in continuation method (4)
%          inits -- maximum # of iterations in ascent/descent method (40)
% Output: u,v    -- solution of primal problem (l1tv reconstruction)
%
% Christian Clason (christian.clason@uni-graz.at)
% Bangti Jin       (btjin@informatik.uni-bremen.de)
% August 14, 2009

%% set parameters
[n,m] = size(f);

% default parameters
if nargin <= 4 || isempty(opts)
    mu     = 1;      % starting value for continuation scheme
    q      = 1/10;   % reduction of mu in continuation scheme
    outits = 4;      % maximum # iterations in continuation scheme
    inits  = 40;     % maximum # iterations in ascent/descent scheme
else
    mu     = opts.mu;
    q      = opts.q;
    outits = opts.outits;
    inits  = opts.inits;
end

% initialize iterates q=(q1,q2), p
q1 = zeros(n,m);
q2 = zeros(n,m);
p  = zeros(n,m);
% initialize leading points
qb1 = q1; qb2 = q2; pb = p;
% initialize Lagrange multiplier, leading point
l = 0; lb = l;

%% compute dual minimizer

% continuation for splitting (mu)
for outit=1:outits
    
    % step lengths for gradient ascent/descent
    gp = 1/mu;            % (reciprocal) Lipschitz constant for mu*K^*
    gq = 1/(8*mu);        % (reciprocal) Lipschitz constant for mu*div
    gl = 0.9*mu;         
    
    % dual ascent/descent method iteration
    for iter=1:inits
        
        % precompute -div q, K^* p (for leading points)
        dx = [qb1(:,m)-qb1(:,1), -diff(qb1,1,2)]+...
             [qb2(n,:)-qb2(1,:); -diff(qb2,1,1)];
        ktx = K(pb);
        
        % compute gradient of augmented Lagrangian
        tmp1 = lb + mu*(3/2*dx+ktx);
        tmp2 = lb + mu*(3/2*ktx+dx);
        eta1 = [diff(tmp1,1,2), tmp1(:,1)-tmp1(:,m)];
        eta2 = [diff(tmp1,1,1); tmp1(1,:)-tmp1(n,:)];
        eta3 = K(tmp2)-f;
        eta4 = ktx+dx;
        
        % update iterates ...
        y1 = q1 - gq * eta1;
        y2 = q2 - gq * eta2;
        y3 = p  - gp * eta3;
        
        % ... and project on {|q|_2<=\alpha, |p|<=1}
        my = (1/alpha)*max(alpha,vnorm(y1,y2));
        q1 = y1./my;
        q2 = y2./my;
        p  = y3./max(1,abs(y3));
        
        % update leading points ...
        y1 = q1 - gq * eta1;
        y2 = q2 - gq * eta2;
        y3 = p  - gp * eta3;
        
        % ... and project
        my  = (1/alpha)*max(alpha,vnorm(y1,y2));
        qb1 = y1./my;
        qb2 = y2./my;
        pb  = y3./max(1,abs(y3));
        
        % update Langrange multiplier, leading point
        l  = l + gl   * eta4;
        lb = l + gl/2 * eta4;
        
    end
    % reduce mu
    mu = mu*q;
end

%% compute primal minimizer

dx  = [q1(:,m)-q1(:,1), -diff(q1,1,2)]+...
      [q2(n,:)-q2(1,:); -diff(q2,1,1)];
ktx = K(p);

u   = l + mu/2*ktx;
v   = l + mu/2*dx;
