% L1TV_TEST
% test splitting method for L1TV formulation with automatic choice of the
% regularization parameter using balancing principle
%
% Christian Clason (christian.clason@uni-graz.at)
% Bangti Jin       (btjin@informatik.uni-bremen.de)
% August 14, 2009

%% set parameters
% problem parameters
n         = 256;          % image size
noisetype = 'saltpepper'; % 'saltpepper' or 'gaussian'
d_per     = 0.4;          % percentage of noise
tvtype    = 'iso';        % 'iso' or 'aniso'
% parameters for balancing fixed point iteration
b_opt.alpha0 = 1;         % starting value for fixed point iteration
%b_opt.sigma  = 1.01;     % balancing parameter (set below)
b_opt.maxits = 10;        % maximum number of fixed point iterations
b_opt.tol    = 1e-2;      % termination criterion: relative change in alpha
% parameters for splitting-dual ascent/descent method
s_opt.mu     = 1;         % starting value for continuation scheme
s_opt.q      = 1/10;      % reduction of mu in continuation scheme
s_opt.outits = 4;         % maximum # iterations in continuation scheme
s_opt.inits  = 40;        % maximum # iterations in ascent/descent scheme

%% academic test image (ramp); replace with your image (scaled to [0,1])
img = zeros(n,n);
img((n/4+1):(3*n/4),(n/4+1):(3*n/4)) = 1;
img((n/4+1):(3*n/4),1:(n/4)) = kron((1:(n/4))/(n/4),ones(n/2,1));

[n,m] = size(img); N = n*m; % image size

%% blurring operator
kernel = fspecial('gaussian',[7 7], 5);
K = @(x) imfilter(x, kernel, 'circular', 'conv');

% type of TV 
if strcmp(tvtype,'aniso')
    vnorm = @(x1,x2) max(abs(x1),abs(x2));
elseif strcmp(tvtype,'iso')
    vnorm = @(x1,x2) sqrt(x1.^2+x2.^2);
end

%% create corrupted image

% blurred image
imb = K(img);

% add noise (salt-and-pepper or Gaussian random-valued)
if strcmp(noisetype,'saltpepper')
    b_opt.sigma = 1.01;
    f = imnoise(imb, 'salt & pepper',d_per);
elseif strcmp(noisetype,'gaussian')
    b_opt.sigma = 1.04;
    drnd   = rand(n,m);
    ind    = find(drnd<d_per);
    imprnd = zeros(n,m);
    imprnd(ind) = randn(length(ind),1);
    noise  = .5*max(abs(img(:)))*abs(imprnd);
    f = imb+noise;
end

% noise level
delta = norm(f(:)-imb(:),1)/N;

display(sprintf('Noise level: delta = %g',delta));

% show noisy, exact image
figure(1), imshow(f,[]);  title('noisy image');
figure(2), imshow(img,[]);title('true image'); 

%% parameter choice and reconstruction

% compute regularization parameter
[u,alpha,delta_est] = l1tv_balancing(f,K,vnorm,b_opt,s_opt);

% (re)compute reconstruction using balancing alpha
tic;
[u,v] = l1tv_solve(f,K,vnorm,alpha,s_opt);
t = toc;

%% evaluate results

% error measures
relerr = norm(u(:)-img(:),2)/norm(img(:),2);
SNR    = 20*log10(norm(img(:)-mean(img(:)))/norm(img(:)-u(:)));
PSNR   = -20*log10(norm(img(:)-u(:))/sqrt(N)) ;

du   = vnorm([diff(u,1,2), u(:,1)-u(:,m)],...
             [diff(u,1,1); u(1,:)-u(n,:)]);
ku   = K(u);
fval = norm(ku(:)-f(:),1)/N + alpha * norm(du(:),1)/N; % functional value

display(sprintf('Reconstruction: relerr = %g, SNR = %.2f, PSNR = %.2f, Fval = %g, time = %g',...
    relerr,SNR,PSNR,fval,t));

% show restored image
figure(3), imshow(u,[]);title('restored image'); 
